#include <iostream>
#include "itemarrayinput.hh"

ItemarrayFileInput::ItemarrayFileInput(char* fn, int minsupp, int maxsupp) : supps(0), remsupps(0), frequent(0),fin(fn) {
  this->minsupp=minsupp; this->maxsupp=maxsupp; 
  this->totsupp=0; this->remsupp=0;

  if(minsupp>0 || maxsupp>0) initSupps();
}

void ItemarrayFileInput::initSupps() {
  int minsupp=this->minsupp;
  this->minsupp=0;
  
  pair<Itemarray*,int>* in;
  
  while(in=getItemarray()) {
    for(int i=0; i<(in->first)->n; ++i) {
      if(supps.size()<=(in->first)->items[i])
	supps.resize((in->first)->items[i]+1);
      
      supps[(in->first)->items[i]]+=in->second;
    }
    
    totsupp+=in->second;
    
    delete in->first; 
    delete in;
  }
  
  this->minsupp=minsupp;
  this->remsupp=totsupp;
  
  remsupps.resize(supps.size());
  frequent.resize(supps.size());
  
  for(int i=0; i<supps.size(); ++i) {
    frequent[i]=supps[i]>=minsupp;
    remsupps[i]=supps[i];
  }

  maxsupp=totsupp-maxsupp;
}


pair<Itemarray *,int>* ItemarrayFileInput::getItemarray() {
  if(fin.eof()) {
    if(minsupp>0)
      for(int i=0; i<supps.size(); ++i) remsupps[i]=supps[i];
    remsupp=totsupp;

    fin.clear();
    fin.seekg(0,ios_base::beg);

    return 0;
  }

  set<Item> itemset;

  int supp=1;

  char c;
  while((c=fin.get())!='\n') {
    if(c=='(') { fin >> supp; fin.get(); } 
    else if(c>='0' && c<='9') {
      fin.unget();
      Item item; fin >> item;
      itemset.insert(item);
    }
  }

  fin.get(); fin.unget();

  Itemarray *out=new Itemarray(itemset.size());

  out->n=0;
  for(set<Item>::const_iterator j=itemset.begin(); j!=itemset.end(); ++j)
    if(minsupp==0 || frequent[*j]) out->items[out->n++]=*j;
  
  if(minsupp>0) {
      remsupp-=supp;
      for(int i=0; i<out->n; ++i) remsupps[out->items[i]]-=supp;
  }
  return new pair<Itemarray*,int>(out,supp);
}

ItemarrayMemoryInput::ItemarrayMemoryInput(char* fn, int minsupp, int maxsupp) : itemarrays(), supps(0), remsupps(0), frequent(0) {
  ItemarrayFileInput fin(fn,minsupp,maxsupp);

  this->totsupp=fin.totsupp;
  this->remsupp=fin.remsupp;
  this->minsupp=fin.minsupp;
  this->maxsupp=fin.maxsupp;

  supps.resize(fin.supps.size());
  remsupps.resize(fin.remsupps.size());

  for(int i=0; i<supps.size(); ++i) {
    supps[i]=fin.supps[i];
    remsupps[i]=fin.remsupps[i];
  }

  pair<Itemarray*,int>* in;
  
  while(in=fin.getItemarray()) {
    iterator i=itemarrays.find(in->first);
    itemarrays[in->first]+=in->second;
    if(i!=itemarrays.end()) delete in->first; 
    delete in;
  }

  cur=itemarrays.begin();
}

pair<Itemarray *,int>* ItemarrayMemoryInput::getItemarray() {
  if(cur==itemarrays.end()) {
    if(minsupp>0)
      for(int i=0; i<supps.size(); ++i) remsupps[i]=supps[i];
    remsupp=totsupp;

    cur=itemarrays.begin();

    return 0;
  }

  Itemarray* out=new Itemarray((*cur).first->n);
  int supp=(*cur).second;

  for(int i=0; i<out->n; ++i) out->items[i]=(*cur).first->items[i];

  if(minsupp>0) {
      remsupp-=supp;
      for(int i=0; i<out->n; ++i) remsupps[out->items[i]]-=supp;
  }

  ++cur;

  return new pair<Itemarray*,int>(out,supp);
}


